VERSION 5.00
Begin VB.UserControl LED 
   CanGetFocus     =   0   'False
   ClientHeight    =   675
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   675
   ScaleHeight     =   45
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   45
   ToolboxBitmap   =   "LED.ctx":0000
   Begin VB.Timer BlinkTimer 
      Enabled         =   0   'False
      Left            =   120
      Top             =   120
   End
End
Attribute VB_Name = "LED"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' LED Display Indicator Control
' Written by:
'   Michael Karas
'   Carousel Design Solutions
'   www.CarouselDesign.com
'
' You may incorporate all or parts of this program into your own project.
' Keep in mind that you got this code free and as such Carousel Design Solutions
' has no responsibility to you, your customers, or anyone else regarding the use
' of this program code. No support or warrenty of any kind can be provided. All
' liability falls upon the user of this code and thus you must judge the suitability
' of this code to your application and usage.
'
' This control is an LED indicator control that can be placed on a form
' to simulate an actual LED. The display indicator is painted as a round
' image using direct drawing commands instead of the more common LED implementations
' using bitmap images. The indicator image includes a 3D border and a reflection
' glint to give the appearance of realism to the control. Through the use of the
' graphic drawing algorithm used the LED can be set to almost any practical size
' at design time from a few pixels across to few hundred pixels, although the
' realism look of the indicator start to degrade after the control is sized over
' about 50 or 60 pizels. (The ability to size the LED was a key motivator to design
' and build this control as other sample controls from the WEB are fixed size and
' many times too small for display screens with a high density pixels/inch setting.
'
' This control supports four property settings all of which are read/write at both
' design time and run time. The properties are:
'
'   LED.State As Boolean                Controls the LED On/Off state. True = On while
'                                       False = Off. (Note that this state is a higher
'                                       state over the Blink states on.off that flashes
'                                       the image.
'
'   LED.Blink As Boolean                This controls whether the LED will blink when it
'                                       is in the On state. An LED can have its blink state
'                                       set on even though the LED may be actually off via
'                                       the State property. True = Blink enabled while
'                                       False = no blink.
'
'   LED.Interval As Integer             This property specifies the blink rate interval for
'                                       the LED in nominal units of milliseconds. Note that
'                                       this property value is utilized directly by a standard
'                                       timer and as such has similar granualrity and resolution
'                                       behavior that a normal timer control will have. Practical
'                                       values for the interval parameter are from 50 to 1000.
'                                       At 1000 the LED blinks off for 1 second and then on for
'                                       1 second.
'
'   LED.Color As LEDColorSelection      This property specifies the color for the LED. The values
'                                       are specified by as Public enumeration value in the control.
'                                       The selection values are:
'
'                                               0 - LED_Red
'                                               1 - LED_Green
'                                               2 - LED_Yellow
'                                               3 - LED_Blue
'
'

Option Explicit

' specify the LED Colors that are supported
Public Enum LEDColorSelection
    LED_Red
    LED_Green
    LED_Yellow
    LED_Blue
End Enum

'
' declare the constants that represent the default values of the properties
' in the user control
'
Private dState As Boolean               ' default the on/off state of the LED
Private dBlink As Boolean               ' default control that indicates if an LED blinks
Private dInterval As Integer            ' default blink interval property
Private dColor As LEDColorSelection     ' default color selection for the LED

' local copies of the controls properties
Private lState As Boolean               ' the on/off state of the LED
Private lBlink As Boolean               ' control that indicates if an LED blinks
Private lInterval As Integer            ' blink interval property
Private lColor As LEDColorSelection     ' color selection for the LED
Private lOnOff As Boolean               ' tracks the actual on/off state of LED

'
' property value routines for getting and setting the states of the user control
'

' property handlers for the State Property
Public Property Get State() As Boolean
Attribute State.VB_Description = "Returns/sets a boolean flag that controls the on/off state of the LED."
    
    State = lState

End Property

Public Property Let State(ByVal NewState As Boolean)
    
    lState = NewState                   ' set the new state value
    lOnOff = NewState
    If lState = False Then              ' this logic sorts out the interaction
        BlinkTimer.Interval = 0         ' of the other control properties in relation
        BlinkTimer.Enabled = False      ' to the On/Off state the blink state and the
    Else                                ' blink timer.
        If lBlink = True Then
            BlinkTimer.Interval = lInterval
            BlinkTimer.Enabled = True
        End If
    End If
    PropertyChanged "State"
    UserControl_Paint                   ' update the LED image

End Property

' property handlers for the Blink Property
Public Property Get Blink() As Boolean
Attribute Blink.VB_Description = "Returns/sets a boolean flag that determines if the LED will blink when it is in the on state."
    
    Blink = lBlink

End Property

Public Property Let Blink(ByVal NewBlink As Boolean)
    
    lBlink = NewBlink                   ' set the new blink state value
    PropertyChanged "Blink"
    
    If lState = True Then                       ' this logic sorts out the interaction
        lOnOff = True                           ' of the other control properties in relation
        If lBlink = True Then                   ' to the On/Off state the blink state and the
            BlinkTimer.Interval = lInterval     ' blink timer.
            BlinkTimer.Enabled = True
        Else
            BlinkTimer.Interval = 0
            BlinkTimer.Enabled = False
        End If
        UserControl_Paint                   ' update the LED image
    End If

End Property

' property handlers for the Interval Property
Public Property Get Interval() As Integer
Attribute Interval.VB_Description = "Returns/sets the blink on and off time interval specified in milliseconds. "
    
    Interval = lInterval

End Property

Public Property Let Interval(ByVal NewInterval As Integer)
    
    lInterval = NewInterval                     ' set the new interval value
    PropertyChanged "Interval"
    
    If BlinkTimer.Enabled = True Then           ' Copy the interval to the blink
        BlinkTimer.Enabled = False              ' timer if the timer is active for
        BlinkTimer.Interval = lInterval         ' the disable / enable is to sync to
        BlinkTimer.Enabled = True               ' interval right away.
    End If

End Property

' property handlers for the Color Property
Public Property Get Color() As LEDColorSelection
Attribute Color.VB_Description = "Returns/sets the color of the LED from selection values."
    
    Color = lColor

End Property

Public Property Let Color(ByVal NewColor As LEDColorSelection)
    
    lColor = NewColor                   ' select the new color setting
    PropertyChanged "Color"
    UserControl_Paint                   ' update the LED image

End Property

' Initialize the default states of the controls properties
' this routine sets up some default value copies of the
' properties so that the Write and Read properties methods
' use the same default values as set here.
Private Sub UserControl_InitProperties()
    
    dState = False
    State = dState
    
    dBlink = False
    Blink = dBlink
    
    dInterval = 0
    Interval = dInterval
    
    dColor = LED_Red
    Color = dColor

End Sub

' routine to support the saving of the current properties values into
' the controls host container property bag.
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
    
    PropBag.WriteProperty "State", State, dState
    PropBag.WriteProperty "Blink", Blink, dBlink
    PropBag.WriteProperty "Interval", Interval, dInterval
    PropBag.WriteProperty "Color", Color, dColor

End Sub

' routine to support the getting of the current properties values from
' the controls host container property bag
Private Sub UserControl_ReadProperties(PropBag As PropertyBag)

    State = PropBag.ReadProperty("State", dState)
    Blink = PropBag.ReadProperty("Blink", dBlink)
    Interval = PropBag.ReadProperty("Interval", dInterval)
    Color = PropBag.ReadProperty("Color", dColor)

End Sub

' initialize default states of the control properties
Private Sub UserControl_Initialize()

End Sub

' routine to respond to the change of the LED blinking and toggle
' its actual on/off state

Private Sub BlinkTimer_Timer()

    If lOnOff = False Then
        lOnOff = True
    Else
        lOnOff = False
    End If

    UserControl_Paint

End Sub

' display the LED image. this paints the image from the actual
' internal lOnOff boolean state so that this represents the
' actual displayed state whilest in the blinking state

Private Sub UserControl_Paint()

    Dim XPos As Integer                 ' X center position for the LED image
    Dim YPos As Integer                 ' Y center position for the LED image
    Dim R As Long
    Dim RBeg As Long                    ' Beginning radius of the LED glint
    Dim RWid As Long                    ' Width if the LED glint
    Dim RSiz As Long                    ' nominal radius of the LED image
    Dim RColor As Long                  ' variable that holds the LED image color
    Dim GColor As Long                  ' variable that holds the LED  glint color
    
    Const PI = 3.1415926
    
    XPos = (ScaleWidth \ 2)             ' compute X center across width of the control
    If XPos > 0 Then                    ' window.
        XPos = XPos - 1
    End If
    YPos = (ScaleHeight \ 2)            ' compute the Y center across the height of the
    If YPos > 0 Then                    ' control window
        YPos = YPos - 1
    End If
    RSiz = ScaleWidth                   ' compute the LED radius value from the smaller
    If RSiz > ScaleHeight Then          ' of the control width and height.
        RSiz = ScaleHeight
    End If
    RSiz = (RSiz \ 2) - 2
    
    RBeg = LInt(RSiz, 5, 2, 19, 4)       ' interpolate / extrapolate a proportional size
    RWid = LInt(RSiz, 5, 1, 19, 4)       ' for the LED glint based on the image raduis

    ' setup the color variables
    Select Case lColor
    
    Case LED_Red
        If lOnOff = False Then
            RColor = &H52&
            GColor = &H8000000F
        Else
            RColor = &HFF&
            GColor = vbWhite
        End If
    
    Case LED_Green
        If lOnOff = False Then
            RColor = &H5200&
            GColor = &H8000000F
        Else
            RColor = &HFF00&
            GColor = vbWhite
        End If
    
    Case LED_Yellow
        If lOnOff = False Then
            RColor = &H8484&
            GColor = &H8000000F
        Else
            RColor = &HFFFF&
            GColor = vbWhite
        End If
    
    Case LED_Blue
        If lOnOff = False Then
            RColor = &H8B0200
            GColor = &H8000000F
        Else
            RColor = &HFF1985
            GColor = vbWhite
        End If
        
    Case Else                   ' default to Red
        If lOnOff = False Then
            RColor = &H5200&
            GColor = &H8000000F
        Else
            RColor = &HFF00&
            GColor = vbWhite
        End If
    
    End Select

    ' draw the 3D border of the LED
    FillStyle = vbFSTransparent
    For R = RSiz + 1 To RSiz Step -1
        Circle (XPos, YPos), R, &H80000010, (45 * PI) / 180, (225 * PI) / 180    ' button shadow
        Circle (XPos, YPos), R, &H80000014, (225 * PI) / 180, (44 * PI) / 180    ' button hilight
    Next R
    
    ' draw the filled circle of the LED at its current color
    FillStyle = vbFSSolid
    FillColor = RColor
    Circle (XPos, YPos), RSiz - 1, RColor
    FillStyle = vbFSTransparent
    
    ' draw the LED glint over the top left side of the image
    For R = RSiz - RBeg To RSiz - RBeg - RWid Step -1
        Circle (XPos, YPos), R, GColor, (90 * PI) / 180, (180 * PI) / 180 ' button hilight
    Next R

End Sub

' function to perform linear interpolation along a straight line segment
' this computes Y from an X value between two known reference points X1.Y1 and X2.Y2
Private Function LInt(X As Long, X1 As Long, Y1 As Long, X2 As Long, Y2 As Long) As Long

    LInt = (((Y2 - Y1) * (X - X1)) / (X2 - X1)) + Y1

End Function
